///////////////////////////////////////////////////////////////////////////////////////////////////
// Company: <Name>
//
// File: bogdan_sm.v
//    Mircea Bogdan
//   September 12, 2020  
//      
//      
//
// 
//
// 
//
// 
//
//
/////////////////////////////////////////////////////////////////////////////////////////////////// 

//`timescale <time_units> / <precision>

module bodgan_sm( 
   input clk,
   input reset,
   input  pg0, pg1, pg2, pg3, pg4,
   output reg [3:0] smstate,
   output reg [4:0] outstate
 );

// state machine definition
parameter [3:0] STATE_0 = 4'b0000;
parameter [3:0] STATE_1 = 4'b0001;
parameter [3:0] STATE_2 = 4'b0010;
parameter [3:0] STATE_3 = 4'b0011;
parameter [3:0] STATE_4 = 4'b0100;
parameter [3:0] STATE_5 = 4'b0101;
parameter [3:0] STATE_6 = 4'b0110;
parameter [3:0] STATE_7 = 4'b0111;
parameter [3:0] STATE_8 = 4'b1000;
parameter [3:0] STATE_9 = 4'b1001;
parameter [3:0] STATE_10 = 4'b1010;

reg [3:0]   state;
reg [31:0]  count;

//might be a good idea to register the inputs
reg  pg0_reg, pg1_reg, pg2_reg, pg3_reg, pg4_reg;
always @ (posedge clk)
begin
	pg0_reg <= pg0;
	pg1_reg <= pg1;
	pg2_reg <= pg2;
	pg3_reg <= pg3;	
	pg4_reg <= pg4;	
	smstate <= state;
end

// here start the state machine
always @ (posedge clk or posedge reset) 
begin
   if (reset == 1'b1)
   begin
      outstate  <= 5'b00000;
	  count	    <= 32'b0;	
      state		<= STATE_0;
   end
   
   else
   
   begin
   case (state)

   STATE_0:
   begin
      outstate  <= 5'b00000;
      count	    <= 32'b0;	
      if(pg0_reg) state <= STATE_1;
   end
	
   STATE_1:
   begin
      outstate  <= 5'b10000;
      count 	<= count + 1'b1;
      if (count > 32'd1400000) // wait for 14ms in this state before decision
      begin
         if (pg1_reg)
         begin
            count <= 32'b0;
            state <= STATE_2;
         end
         else 		 
         begin
            count <= 32'b0;
            state <= STATE_10;
         end
      end
   end
	
   STATE_2:
   begin
      outstate  <= 5'b11000;
      count 	<= count + 1'b1;
      if (count > 32'd400000)  // wait for 4ms in this state before decision
      begin
         if (pg2_reg)
         begin
            count <= 32'b0;
            state <= STATE_3;
         end
         else 		 
         begin
            count <= 32'b0;
            state <= STATE_9;
         end
      end
   end
	
   STATE_3:
   begin
      outstate  <= 5'b11100;
      count 	<= count + 1'b1;
      if (count > 32'd250000)  // wait for 5ms in this state before decision
      begin
         if (pg3_reg)
         begin
            count <= 32'b0;
            state <= STATE_4;
         end
         else 		 
         begin
            count <= 32'b0;
            state <= STATE_8;
         end
      end
   end

   STATE_4:
   begin
      outstate  <= 5'b11110;
      count 	<= count + 1'b1;
      if (count > 32'd500000)  // wait for 10ms in this state before decision
      begin
         if (pg4_reg)
         begin
            count <= 32'b0;
            state <= STATE_5;
         end
         else 		 
         begin
            count <= 32'b0;
            state <= STATE_7;
         end
      end
   end
   
   STATE_5:
   begin
      outstate  <= 5'b11111;
      count 	<= 32'b0;
      if (!(pg0_reg || pg1_reg || pg2_reg || pg3_reg || pg4_reg)) state <= STATE_6;
   end

   STATE_6:
   begin
      outstate  <= 5'b11100;
      count 	<= count + 1'b1;
      if (count > 32'd5000)  // wait for 0.1ms in this state before decision
      begin
         count <= 32'b0;
         state <= STATE_7;
      end
   end

   STATE_7:
   begin
      outstate  <= 5'b11000;
      count 	<= count + 1'b1;
      if (count > 32'd75000)  // wait for 1.5ms in this state before decision
      begin
         count <= 32'b0;
         state <= STATE_8;
      end
   end

   STATE_8:
   begin
      outstate  <= 5'b10000;
      count 	<= count + 1'b1;
      if (count > 32'd20000)  // wait for 0.4ms in this state before decision
      begin
         count <= 32'b0;
         state <= STATE_9;
      end
   end

   STATE_9:
   begin
      outstate  <= 5'b00000;
      count 	<= count + 1'b1;
      if (count > 32'd25000)  // wait for 0.5ms in this state before decision
      begin
         count <= 32'b0;
         state <= STATE_10;
      end
   end

   STATE_10:
   begin
      outstate  <= 5'b00000;
      count 	<= 32'b0;
   end

   default:
   begin
      outstate  <= 5'b00000;
      count	    <= 32'b0;
      state     <= STATE_0; 
   end
 
   endcase
   end
end

endmodule

